/* Copyright (c) 1992 by Michael J. Roberts.  All Rights Reserved. */
/*
Name
  ossmac.h - definitions for Mac OS support routines.
Function
  These functions are used only in Mac-specific code, so they
  are broken out from osmac.h, which contains definitions for
  cross-platform functions.
Notes
  None
Modified
  10/25/97 CNebel        - creation
*/

#ifndef __OSSMAC__
#define __OSSMAC__

#include <stddef.h>
#include <stdarg.h>

#include <Types.h>
#include <Quickdraw.h>
#include <MacMemory.h>
#include <Files.h>
#include <Events.h>
#include <Dialogs.h>
#include <Lists.h>

#include "mdebug.h"
#include "mstring.h"
#include "argize.h"


/*
 * Constants for linefeed and carriage return that are immune to compiler switches.
 */
#define kLF '\012'
#define kCR '\015'

/*
 * QuickDraw utility functions.
 */
#ifdef __cplusplus
	inline short Dx(const Rect &r) { return r.right - r.left; }
	inline short Dy(const Rect &r) { return r.bottom - r.top; }
#else
	#define Dx(r) ((r).right - (r).left)
	#define Dy(r) ((r).bottom - (r).top)
#endif

/*
 * State-saving objects.
 */
#ifdef __cplusplus
class TPortState {
	private:
		GrafPtr	fPort;
	public:
		TPortState()				{ GetPort(&fPort); }
		TPortState(GrafPtr newPort)	{ GetPort(&fPort); SetPort(newPort); }
		~TPortState()				{ SetPort(fPort); }
};

class TFontState {
	private:
		#ifndef NDEBUG
		GrafPtr	fPort;
		#endif
		short	fFont, fSize, fMode;
		Style	fFace;
	public:
		TFontState();
		~TFontState();
};

class TFractEnableState {
	private:
		UInt8	fEnable;
	public:
		TFractEnableState();
		TFractEnableState(UInt8 enable);
		~TFractEnableState();
};

class THState {
	private:
		Handle	fh;
		SInt8	fHState;
	public:
		THState(Handle h) : fh(h)	{ if (h) fHState = HGetState(h); }
		~THState()					{ if (fh) HSetState(fh, fHState); }
};

class THLock {
	private:
		THState	fHState;
	public:
		THLock(Handle h) : fHState(h) { if (h) HLock(h); }
};

class THUnlock {
	private:
		THState	fHState;
	public:
		THUnlock(Handle h) : fHState(h) { if (h) HUnlock(h); }
};

#endif


/*
 *	Toolbox code.
 */

extern OSType	gAppCreator;
extern short	gAppResFile;

OSErr InitToolbox(void);  // Sets gAppResFile.
OSErr GetApplicationInfo(StringPtr name, FSSpec *fsspec);  // Sets gAppCreator.
OSErr CloseToolbox(void);

void HandleAllHighLevelEvents();

// HandleAllHighLevelEvents calls the following routines which
// must be defined by the client app:
OSErr HandleOpenApp();
OSErr HandleOpenDocument(const FSSpec *doc);
void  HandleQuitApp();


/*
 *  File routines.
 */
OSErr os0askfile(const char *prompt, Str255 fname, OSType type, bool wrtflag);
OSErr os0askfile(const char *prompt, char *fnp, size_t fnl, OSType type, bool wrtflag);
OSErr osaskdir(char *fnp, int fnl);

OSErr DirToPath(short vRefNum, long dirID, Str255 s);
OSErr dirtopath(short vRefNum, long dirID, char *s);
OSErr VRefToPath(short vRef, Str255 s);
OSErr vreftopath(short vRef, char *s);
OSErr FSSpecToPath(const FSSpec *fsspec, Str255 s);
OSErr fsspectopath(const FSSpec *fsspec, char *s);


/*
 *  Dialog utilities.
 */

#ifndef __APPEARANCE__
pascal OSErr GetDialogItemAsControl(DialogPtr inDialog, SInt16 inItemNo, ControlHandle *outControl);
#endif

SInt16 GetControlItemValue(DialogPtr dialog, short itemNo);
void SetControlItemValue(DialogPtr dialog, short item, SInt16 newvalue);
void GetControlItemTitle(DialogPtr dialog, short itemNo, Str255 title);
void SetControlItemTitle(DialogPtr dialog, short itemNo, ConstStr255Param title);
void GetControlItemRect(DialogPtr dialog, short itemNo, Rect *bounds);
void HiliteControlItem(DialogPtr dialog, short itemNo, ControlPartCode hiliteState);
#ifdef __cplusplus
inline void EnableControlItem(DialogPtr dialog, short itemNo)
	{ HiliteControlItem(dialog, itemNo, 0); }
inline void DisableControlItem(DialogPtr dialog, short itemNo)
	{ HiliteControlItem(dialog, itemNo, 255); }
#else
#define EnableControlItem(d, i)  HiliteControlItem((d), (i), 0)
#define DisableControlItem(d, i)  HiliteControlItem((d), (i), 255)
#endif

bool ToggleControlItemValue(DialogPtr dialog, short itemNo);
void drawbox(DialogPtr dialog, short itemNo);


/*
 *  This routine exists to fix a weird library bug.  Try to remove it soon...
 */
void XXX_FixStandAlone(void);


/*
 *  Miscellaneous crap.
 */
#define kScrollBarWidth		15

void SetUpWindows(void);
void UpdateWindow(WindowPtr theWindow);
void MyGrowWindow(WindowPtr w, Point p);
bool ours(WindowPtr w);
int DoContent(WindowPtr theWindow, EventRecord *theEvent);
int topEvent(DialogPtr mydlg, ListHandle mylist);

void os_putln(char *s);

/* Called only from osmac.c */
void os_update(void);
void editinit(void);
void macedDbPut(char *s);
void macedShostat(int doloc /* 1 if location is to be redrawn, 0 otherwise */);

#endif
